<#--  Import common macros and functions  -->
<#import "common_module_traversal_utils.ftl" as utils>
<#import "macros_common_general.ftl" as com>
<#compress>
    <#--  Initialize common variables  -->
    <@com.initializeMainVariables/>

    <#--  Root entity  -->
    <#if subEntityUuid??>
        <#assign rootEntity = com.getSubEntityKey(subEntityUuid) />
    <#else>
        <#assign rootEntity = entity.root />
    </#if>

    <#--  Define the main macro and the arguments to be used to get output using the traversal approach  -->
    <#assign mainMacroName = "addTestMaterialsForDocument"/>
    <#assign mainMacroArgs = ['sectionDoc', 'level']/>

    <#--  Make global the macroCall as well as each macro used in it  -->
    <#global mainMacroCall="<@" + mainMacroName + " " + mainMacroArgs?join(" ") + "/>"/>
    <#global addTestMaterialsForDocument=addTestMaterialsForDocument/>

    <#--  
        Initialize hash to store test materials information (input to table).
        It's a hash of lists of hashes, which means it's a collection of key-value pairs, where the 
        values are themselves lists of hashes.
        The outer hash, testMaterialsHash, is used to store information about test materials. Each key 
        in this outer hash corresponds to the name or identifier of a test material, and the associated 
        value is a list of hashes.
        The inner hash, which is associated with each test material, stores detailed information about 
        that test material. It typically contains keys like "type," "refSub," and "conc," each of which 
        holds specific information about the composition of the tst material, such as its type, reference 
        substance, and concentration.
    -->
    <#assign testMaterialsHash = {}/>

    <#-- Do the traverse -->
    <@utils.traversal rootEntity 8/>
    
    <#--  Table output  -->
	<@tableOutput testMaterialsHash/>


<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->
<#macro printCsvHeadersRow hash>
    <#-- Initialize a sequence to keep track of unique reference substances -->
    <#assign uniqueRefSubs = []>

    <#--  Iterate over hash keys  -->
    <#list hash?keys as testMaterial>
        <#--  Iterate over Test Material's composition  -->
        <#list hash[testMaterial] as comp>
            <#--  Append reference substance to unique reference substances  -->
            <#if !(uniqueRefSubs?seq_contains(comp.refSub))>
                <#assign uniqueRefSubs = uniqueRefSubs + [comp.refSub] />
            </#if>
        </#list>
    </#list>
    "Test Material",${uniqueRefSubs?join(",")}
</#macro>

<#--
    This macro is used to extract information about test materials from a document.

    Arguments:
    - sectionDoc: The current section document being processed.
    - sectionNode: The current section node.
    - entityDocument: The document entity.
    - level: The level of the document section.

    This macro is typically called as part of a traversal process to gather information about test 
    materials within the document.

    It performs the following steps:
    1. Check if the current level is 0, indicating the top-level section.
    2. Retrieve a list of test materials from the document using the "getTestMaterialsEntitiesFromDoc" 
    function.
    3. If there are test materials in the list, iterate through each test material.
    4. For each test material, create an empty sequence "composition" to store information about 
    its composition.
    5. Iterate through the composition list of the test material and extract data such as reference 
    substance, type, and concentration.
    6. Create a "rowHash" that stores this information.
    7. Append the "rowHash" to the "composition" sequence.
    8. Add an entry to the "testMaterialsHash" using the test material's name as the key and the 
    "composition" sequence as the value.
-->
<#macro addTestMaterialsForDocument sectionDoc level>
	<#if level == 0>
		<#--  Get test materials list from document  -->
  		<#assign testMaterialsList = getTestMaterialsEntitiesFromDoc(sectionDoc) >

        <#--  Check if test materials list has content  -->
    	<#if testMaterialsList?has_content>
            <#--  Iterate over test materials list  -->
      		<#list testMaterialsList as testMaterial>
                <#--  <para>Section name: ${sectionDoc.name}</para>
				<para>Test material: ${testMaterial.Name}</para>  -->

                <#--  Initialize sequence that will hold composition data  -->
                <#local composition = [] />

                <#--  Iterate over test material composition block  -->
                <#list testMaterial.Composition.CompositionList as comp>
                    <#--  Get "Reference Substance", "Type" and "Concentration"  -->
                    <#local refSub = iuclid.getDocumentForKey(comp.ReferenceSubstance) />
                    <#local refSubName><@com.text refSub.ReferenceSubstanceName /></#local>
                    <#local type><@com.picklist comp.Type /></#local>
                    <#local conc><@com.range comp.Concentration /></#local>

                    <#--  Store composition data in hash  -->
                    <#local rowHash = {
                        "refSub" : refSubName,
                        "type" : type,
                        "conc" : conc
                    }/>

                    <#--  Append hash to composition sequence  -->
                    <#local composition = composition + [rowHash] />

                    <#--  <para>Reference Substance: ${refSub.ReferenceSubstanceName}</para>
				    <para>Concentration: ${conc}</para>
                    <@com.emptyLine/>  -->
                </#list>

                <#--  Append composition to "testMaterialsHash" using the test material's name as the key  -->
                <#assign testMaterialsHash = testMaterialsHash + {testMaterial.Name : composition}/>
      		</#list>
    	</#if>
    </#if>
</#macro>

<#--
    This function retrieves all test materials from a given document.

    Arguments:
    - sectionDoc: The current section ocument being processed.

    The function performs the following steps:
    1. Initialize an empty sequence, "testMaterialsList," to store the list of test materials.
    2. Check if the "sectionDoc" is of type "document_reference". If so, it's a direct reference 
    to a document.
       - If the referenced document exists and is of type "TEST_MATERIAL_INFORMATION," add it to 
       the list of test materials.
    3. Check if the "sectionDoc" is of type "document_references". If so, it's a list of document references.
       - Iterate through the list of references, and for each referenced document:
         - If the referenced document exists and is of type "TEST_MATERIAL_INFORMATION," add it to the "testMaterialsList."
    4. If the "sectionDoc" has child sections (subsections), recursively call the function on each child section.
    5. Return the "testMaterialsList" sequence, which contains all the test materials from the document.
-->
<#function getTestMaterialsEntitiesFromDoc sectionDoc>
    <#--  Initialize sequence that will hold list of test materials  -->
	<#local testMaterialsList = [] />

    <#--  Check if section document is a document reference  -->
	<#if sectionDoc?node_type == "document_reference">
        <#--  Get referenced document  -->
        <#local doc = iuclid.getDocumentForKey(sectionDoc) />

        <#--  Check if the referenced document has content and is of type "TEST_MATERIAL_INFORMATION"  -->
        <#if doc?has_content && doc.documentType == "TEST_MATERIAL_INFORMATION">
            <#--  Append referenced document to tlist of test materials  -->
            <#local testMaterialsList = testMaterialsList + [doc] />

            <#--  Return list of test materials  -->
            <#return testMaterialsList />
        </#if>
	</#if>

    <#--  Check if the section document is a list of document references  -->
	<#if sectionDoc?node_type == "document_references">
        <#--  Iterate through list of references  -->
		<#list sectionDoc as item>
            <#--  Get referenced document  -->
            <#local doc = iuclid.getDocumentForKey(item) />

            <#--  Check if the referenced document has content and is of type "TEST_MATERIAL_INFORMATION"  -->
            <#if doc?has_content && doc.documentType == "TEST_MATERIAL_INFORMATION">
                <#--  Append referenced document to tlist of test materials  -->
			    <#local testMaterialsList = testMaterialsList + [doc] />
            </#if>
		</#list>

        <#--  Return list of test materials  -->
		<#return testMaterialsList />
	</#if>

    <#--  Check if section document has child sections (subsections)  -->
	<#if sectionDoc?children?has_content>
        <#--  Iterate through child sections  -->
		<#list sectionDoc?children as child>
            <#--  Recursively call the function on each child section to retrieve test materials  -->
			<#local testMaterialsList = testMaterialsList + getTestMaterialsEntitiesFromDoc(child)  />
		</#list>
	</#if>

    <#--  Return list of test materials  -->
	<#return testMaterialsList/>
</#function>


<#-- Create table for the test materials -->
<#macro tableOutput hash>
    <#if hash?has_content>
        <#-- Initialize a sequence of sequences to store rows data -->
        <#assign tableRows = []>

        <#-- Initialize a sequence to keep track of unique reference substances -->
        <#assign uniqueRefSubs = []>

        <#--  Iterate over hash keys  -->
        <#list hash?keys as testMaterial>
            <#--  Iterate over Test Material's composition  -->
            <#list hash[testMaterial] as comp>
                <#--  Append reference substance to unique reference substances  -->
                <#if !(uniqueRefSubs?seq_contains(comp.refSub))>
                    <#assign uniqueRefSubs = uniqueRefSubs + [comp.refSub] />
                </#if>
            </#list>
        </#list>

        "Test Material",${uniqueRefSubs?join(",")}

        <#--  Iterate over hash keys  -->
        <#list hash?keys as testMaterial>
            <#assign row = [testMaterial]>

            <#--  ${testMaterial}  -->

            <#--  Unique reference substances  -->
            <#if uniqueRefSubs?size gt 0>
                <#list uniqueRefSubs as refSub>
                    <#assign found = false>

                    <#list hash[testMaterial] as comp>
                        <#if refSub == comp.refSub>
                            <#assign value>
                                <@utils.csvValue comp.conc/>
                            </#assign>

                            <#assign row = row + [value?replace("&gt;", ">")?replace("&lt;", "<")?replace("\n", " ")]>

                            <#assign found = true>
                        </#if>
                    </#list>

                    <#if !found>
                        <#assign row = row + [""]>
                    </#if>
                </#list>

                ${row?join(",")}
            </#if>
        </#list>
    </#if>
</#macro>
</#compress>